@@ -311,7 +311,7 @@ class LikelihoodEstimator(NeuralInference, ABC):
             device = self._device
         else:
             # Otherwise, infer it from the device of the net parameters.
-            device = next(density_estimator.parameters()).device
+            device = next(density_estimator.parameters()).device.type
 
         self._posterior = LikelihoodBasedPosterior(
             method_family="snle",
