@@ -4,20 +4,20 @@
 import logging
 import warnings
 from math import pi
-from typing import Any, Dict, List, Optional, Sequence, Tuple, Union
+from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
 
 import torch
-from pyknos.nflows import transforms
+import pyknos.nflows.transforms as transforms
 from pyro.distributions import Empirical
-from torch import Tensor, as_tensor
+from torch import Tensor, as_tensor, float32
 from torch import nn as nn
-from torch import ones, zeros
-from torch.distributions import Independent
-from torch.distributions.distribution import Distribution
+from torch import ones, optim, zeros
+from torch.distributions import Distribution, Independent, biject_to
+import torch.distributions.transforms as torch_tf
 from tqdm.auto import tqdm
 
 from sbi import utils as utils
-from sbi.utils.torchutils import atleast_2d
+from sbi.utils.torchutils import BoxUniform, atleast_2d
 
 
 def warn_if_zscoring_changes_data(x: Tensor, duplicate_tolerance: float = 0.1) -> None:
@@ -147,20 +147,25 @@ def standardizing_net(batch_t: Tensor, min_std: float = 1e-7) -> nn.Module:
 
 
 @torch.no_grad()
-def sample_posterior_within_prior(
-    posterior_nn: nn.Module,
-    prior,
+def rejection_sample_posterior_within_prior(
+    posterior_nn: Any,
+    prior: Callable,
     x: Tensor,
-    num_samples: int = 1,
+    num_samples: int,
     show_progress_bars: bool = False,
     warn_acceptance: float = 0.01,
     sample_for_correction_factor: bool = False,
     max_sampling_batch_size: int = 10_000,
+    **kwargs,
 ) -> Tuple[Tensor, Tensor]:
     r"""Return samples from a posterior $p(\theta|x)$ only within the prior support.
 
     This is relevant for snpe methods and flows for which the posterior tends to have
-     mass outside the prior boundaries.
+    mass outside the prior support.
+
+    This function could in principle be integrated into `rejection_sample()`. However,
+    to keep the warnings clean, to avoid additional code for integration, and confusing
+    if-cases, we decided to keep two separate functions.
 
     This function uses rejection sampling with samples from posterior in order to
         1) obtain posterior samples within the prior support, and
@@ -176,19 +181,27 @@ def sample_posterior_within_prior(
         warn_acceptance: A minimum acceptance rate under which to warn about slowness.
         sample_for_correction_factor: True if this function was called by
             `leakage_correction()`. False otherwise. Will be used to adapt the leakage
-             warning.
+             warning and to decide whether we have to search for the maximum.
         max_sampling_batch_size: Batch size for drawing samples from the posterior.
             Takes effect only in the second iteration of the loop below, i.e., in case
             of leakage or `num_samples>max_sampling_batch_size`. Larger batch size
             speeds up sampling.
+        kwargs: Absorb additional unused arguments that can be passed to
+            `rejection_sample()`. Warn if not empty.
 
     Returns:
         Accepted samples and acceptance rate as scalar Tensor.
     """
 
-    assert not posterior_nn.training, "Posterior nn must be in eval mode for sampling."
+    if kwargs:
+        logging.warn(
+            f"You passed arguments to `rejection_sampling_parameters` that "
+            f"are unused when you do not specify a `proposal` in the same "
+            f"dictionary. The unused arguments are: {kwargs}"
+        )
 
-    # Progress bar can be skipped, e.g. when sampling after each round just for logging.
+    # Progress bar can be skipped, e.g. when sampling after each round just for
+    # logging.
     pbar = tqdm(
         disable=not show_progress_bars,
         total=num_samples,
@@ -204,13 +217,14 @@ def sample_posterior_within_prior(
     while num_remaining > 0:
 
         # Sample and reject.
-        candidates = (
-            posterior_nn.sample(sampling_batch_size, context=x)
-            .reshape(sampling_batch_size, -1)
-            .cpu()  # Move to cpu to evaluate under prior.
+        candidates = posterior_nn.sample(sampling_batch_size, context=x).reshape(
+            sampling_batch_size, -1
         )
+
+        # SNPE-style rejection-sampling when the proposal is the neural net.
         are_within_prior = within_support(prior, candidates)
         samples = candidates[are_within_prior]
+
         accepted.append(samples)
 
         # Update.
@@ -233,24 +247,26 @@ def sample_posterior_within_prior(
             if sample_for_correction_factor:
                 logging.warning(
                     f"""Drawing samples from posterior to estimate the normalizing
-                        constant for `log_prob()`. However, only {acceptance_rate:.0%}
-                        posterior samples are within the prior support. It may take a
-                        long time to collect the remaining {num_remaining} samples.
-                        Consider interrupting (Ctrl-C) and either basing the estimate
-                        of the normalizing constant on fewer samples (by calling
-                        `posterior.leakage_correction(x_o, num_rejection_samples=N)`,
-                        where `N` is the number of samples you want to base the
+                        constant for `log_prob()`. However, only
+                        {acceptance_rate:.0%} posterior samples are within the
+                        prior support. It may take a long time to collect the
+                        remaining {num_remaining} samples.
+                        Consider interrupting (Ctrl-C) and either basing the
+                        estimate of the normalizing constant on fewer samples (by
+                        calling `posterior.leakage_correction(x_o,
+                        num_rejection_samples=N)`, where `N` is the number of
+                        samples you want to base the
                         estimate on (default N=10000), or not estimating the
                         normalizing constant at all
-                        (`log_prob(..., norm_posterior=False)`. The latter will result
-                        in an unnormalized `log_prob()`."""
+                        (`log_prob(..., norm_posterior=False)`. The latter will
+                        result in an unnormalized `log_prob()`."""
                 )
             else:
                 logging.warning(
                     f"""Only {acceptance_rate:.0%} posterior samples are within the
-                        prior support. It may take a long time to collect the remaining
-                        {num_remaining} samples. Consider interrupting (Ctrl-C)
-                        and switching to `sample_with_mcmc=True`."""
+                        prior support. It may take a long time to collect the
+                        remaining {num_remaining} samples. Consider interrupting
+                        (Ctrl-C) and switching to `sample_with='mcmc'`."""
                 )
             leakage_warning_raised = True  # Ensure warning is raised just once.
 
@@ -265,6 +281,165 @@ def sample_posterior_within_prior(
     return samples, as_tensor(acceptance_rate)
 
 
+def rejection_sample(
+    potential_fn: Callable,
+    proposal: Any,
+    num_samples: int = 1,
+    show_progress_bars: bool = False,
+    warn_acceptance: float = 0.01,
+    max_sampling_batch_size: int = 10_000,
+    num_samples_to_find_max: int = 10_000,
+    num_iter_to_find_max: int = 100,
+    m: float = 1.2,
+) -> Tuple[Tensor, Tensor]:
+    r"""Return samples from a `potential_fn` obtained via rejection sampling.
+
+    This function uses rejection sampling with samples from posterior in order to
+        1) obtain posterior samples within the prior support, and
+        2) calculate the fraction of accepted samples as a proxy for correcting the
+           density during evaluation of the posterior.
+
+    Args:
+        potential_fn: The potential to sample from. The potential should be passed as
+            the logarithm of the desired distribution.
+        proposal: The proposal from which to draw candidate samples. Must have a
+            `sample()` and a `log_prob()` method.
+        num_samples: Desired number of samples.
+        show_progress_bars: Whether to show a progressbar during sampling.
+        warn_acceptance: A minimum acceptance rate under which to warn about slowness.
+        max_sampling_batch_size: Batch size for drawing samples from the posterior.
+            Takes effect only in the second iteration of the loop below, i.e., in case
+            of leakage or `num_samples>max_sampling_batch_size`. Larger batch size
+            speeds up sampling.
+        num_samples_to_find_max: Number of samples that are used to find the maximum
+            of the `potential_fn / proposal` ratio.
+        num_iter_to_find_max: Number of gradient ascent iterations to find the maximum
+            of the `potential_fn / proposal` ratio.
+        m: Multiplier to the maximum ratio between potential function and the
+            proposal. This factor is applied after already having scaled the proposal
+            with the maximum ratio of the `potential_fn / proposal` ratio. A higher
+            value will ensure that the samples are indeed from the correct
+            distribution, but will increase the fraction of rejected samples and thus
+            computation time.
+
+    Returns:
+        Accepted samples and acceptance rate as scalar Tensor.
+    """
+
+    samples_to_find_max = proposal.sample((num_samples_to_find_max,))
+
+    # Define a potential as the ratio between target distribution and proposal.
+    def potential_over_proposal(theta):
+        return potential_fn(theta) - proposal.log_prob(theta)
+
+    # Search for the maximum of the ratio.
+    _, max_log_ratio = optimize_potential_fn(
+        potential_fn=potential_over_proposal,
+        inits=samples_to_find_max,
+        dist_specifying_bounds=proposal,
+        num_iter=num_iter_to_find_max,
+        learning_rate=0.01,
+        num_to_optimize=max(1, int(num_samples_to_find_max / 10)),
+        show_progress_bars=False,
+    )
+
+    if m < 1.0:
+        warnings.warn("A value of m < 1.0 will lead to systematically wrong results.")
+
+    class ScaledProposal:
+        """
+        Proposal for rejection sampling which is strictly larger than the potential_fn.
+        """
+
+        def __init__(self, proposal: Any, max_log_ratio: float, log_m: float):
+            self.proposal = proposal
+            self.max_log_ratio = max_log_ratio
+            self.log_m = log_m
+
+        def sample(self, sample_shape: torch.Size, **kwargs) -> Tensor:
+            """
+            Samples from the `ScaledProposal` are samples from the `proposal`.
+            """
+            return self.proposal.sample((sample_shape,), **kwargs)
+
+        def log_prob(self, theta: Tensor, **kwargs) -> Tensor:
+            """
+            The log-prob is scaled such that the proposal is always above the potential.
+            """
+            return (
+                self.proposal.log_prob(theta, **kwargs)
+                + self.max_log_ratio
+                + self.log_m
+            )
+
+    proposal = ScaledProposal(proposal, max_log_ratio, torch.log(torch.as_tensor(m)))
+
+    with torch.no_grad():
+        # Progress bar can be skipped, e.g. when sampling after each round just for
+        # logging.
+        pbar = tqdm(
+            disable=not show_progress_bars,
+            total=num_samples,
+            desc=f"Drawing {num_samples} posterior samples",
+        )
+
+        num_sampled_total, num_remaining = 0, num_samples
+        accepted, acceptance_rate = [], float("Nan")
+        leakage_warning_raised = False
+
+        # To cover cases with few samples without leakage:
+        sampling_batch_size = min(num_samples, max_sampling_batch_size)
+        while num_remaining > 0:
+
+            # Sample and reject.
+            candidates = proposal.sample(sampling_batch_size).reshape(
+                sampling_batch_size, -1
+            )
+
+            target_proposal_ratio = torch.exp(
+                potential_fn(candidates) - proposal.log_prob(candidates)
+            )
+            uniform_rand = torch.rand(target_proposal_ratio.shape)
+            samples = candidates[target_proposal_ratio > uniform_rand]
+
+            accepted.append(samples)
+
+            # Update.
+            num_sampled_total += sampling_batch_size
+            num_remaining -= samples.shape[0]
+            pbar.update(samples.shape[0])
+
+            # To avoid endless sampling when leakage is high, we raise a warning if the
+            # acceptance rate is too low after the first 1_000 samples.
+            acceptance_rate = (num_samples - num_remaining) / num_sampled_total
+
+            # For remaining iterations (leakage or many samples) continue sampling with
+            # fixed batch size.
+            sampling_batch_size = max_sampling_batch_size
+            if (
+                num_sampled_total > 1000
+                and acceptance_rate < warn_acceptance
+                and not leakage_warning_raised
+            ):
+                logging.warning(
+                    f"""Only {acceptance_rate:.0%} proposal samples were accepted. It
+                        may take a long time to collect the remaining {num_remaining}
+                        samples. Consider interrupting (Ctrl-C) and switching to
+                        `sample_with='mcmc`."""
+                )
+                leakage_warning_raised = True  # Ensure warning is raised just once.
+
+        pbar.close()
+
+        # When in case of leakage a batch size was used there could be too many samples.
+        samples = torch.cat(accepted)[:num_samples]
+        assert (
+            samples.shape[0] == num_samples
+        ), "Number of accepted samples must match required samples."
+
+    return samples, as_tensor(acceptance_rate)
+
+
 def handle_invalid_x(
     x: Tensor, exclude_invalid_x: bool = True
 ) -> Tuple[Tensor, int, int]:
@@ -371,9 +546,7 @@ def check_warn_and_setstate(
 
 
 def get_simulations_since_round(
-    data: List,
-    data_round_indices: List,
-    starting_round_index: int,
+    data: List, data_round_indices: List, starting_round_index: int
 ) -> Tensor:
     """
     Returns tensor with all data coming from a round >= `starting_round`.
@@ -443,6 +616,161 @@ def batched_mixture_mv(matrix: Tensor, vector: Tensor) -> Tensor:
     return torch.einsum("bcij,bcj -> bci", matrix, vector)
 
 
+def optimize_potential_fn(
+    potential_fn: Callable,
+    inits: Tensor,
+    dist_specifying_bounds: Optional[Any] = None,
+    num_iter: int = 1_000,
+    num_to_optimize: int = 100,
+    learning_rate: float = 0.01,
+    save_best_every: int = 10,
+    show_progress_bars: bool = False,
+    interruption_note: str = "",
+) -> Tuple[Tensor, Tensor]:
+    """
+    Returns the `argmax` and `max` of a `potential_fn`.
+
+    The method can be interrupted (Ctrl-C) when the user sees that the log-probability
+    converges. The best estimate will be returned.
+
+    The maximum is obtained by running gradient ascent from given starting parameters.
+    After the optimization is done, we select the parameter set that has the highest
+    `potential_fn` value after the optimization.
+
+    Warning: The default values used by this function are not well-tested. They might
+    require hand-tuning for the problem at hand.
+
+    Args:
+        potential_fn: The function on which to optimize.
+        inits: The initial parameters at which to start the gradient ascent steps.
+        dist_specifying_bounds: Distribution the specifies bounds for the optimization.
+            If it is a `sbi.utils.BoxUniform`, we transform the space into
+            unconstrained space and carry out the optimization there.
+        num_iter: Number of optimization steps that the algorithm takes
+            to find the MAP.
+        num_to_optimize: From the drawn `num_init_samples`, use the `num_to_optimize`
+            with highest log-probability as the initial points for the optimization.
+        learning_rate: Learning rate of the optimizer.
+        save_best_every: The best log-probability is computed, saved in the
+            `map`-attribute, and printed every `save_best_every`-th iteration.
+            Computing the best log-probability creates a significant overhead (thus,
+            the default is `10`.)
+        show_progress_bars: Whether or not to show a progressbar for the optimization.
+        interruption_note: The message printed when the user interrupts the
+            optimization.
+
+    Returns:
+        The `argmax` and `max` of the `potential_fn`.
+    """
+
+    # If the prior is `BoxUniform`, define a transformation to optimize in
+    # unbounded space.
+    if dist_specifying_bounds is not None:
+        is_boxuniform, boxuniform = check_dist_class(dist_specifying_bounds, BoxUniform)
+    else:
+        is_boxuniform = False
+
+    if is_boxuniform:
+
+        def tf_inv(theta_t):
+            return utils.expit(
+                theta_t,
+                torch.as_tensor(
+                    boxuniform.support.base_constraint.lower_bound, dtype=float32
+                ),
+                torch.as_tensor(
+                    boxuniform.support.base_constraint.upper_bound, dtype=float32
+                ),
+            )
+
+        def tf(theta):
+            return utils.logit(
+                theta,
+                torch.as_tensor(
+                    boxuniform.support.base_constraint.lower_bound, dtype=float32
+                ),
+                torch.as_tensor(
+                    boxuniform.support.base_constraint.upper_bound, dtype=float32
+                ),
+            )
+
+    else:
+
+        def tf_inv(theta_t):
+            return theta_t
+
+        def tf(theta):
+            return theta
+
+    init_probs = potential_fn(inits).detach()
+
+    # Pick the `num_to_optimize` best init locations.
+    sort_indices = torch.argsort(init_probs, dim=0)
+    sorted_inits = inits[sort_indices]
+    optimize_inits = sorted_inits[-num_to_optimize:]
+
+    # The `_overall` variables store data accross the iterations, whereas the
+    # `_iter` variables contain data exclusively extracted from the current
+    # iteration.
+    best_log_prob_iter = torch.max(init_probs)
+    best_theta_iter = sorted_inits[-1]
+    best_theta_overall = best_theta_iter.detach().clone()
+    best_log_prob_overall = best_log_prob_iter.detach().clone()
+
+    argmax_ = best_theta_overall
+    max_val = best_log_prob_overall
+
+    optimize_inits = tf(optimize_inits)
+    optimize_inits.requires_grad_(True)
+    optimizer = optim.Adam([optimize_inits], lr=learning_rate)
+
+    iter_ = 0
+
+    # Try-except block in case the user interrupts the program and wants to fall
+    # back on the last saved `.map_`. We want to avoid a long error-message here.
+    try:
+
+        while iter_ < num_iter:
+
+            optimizer.zero_grad()
+            probs = potential_fn(tf_inv(optimize_inits)).squeeze()
+            loss = -probs.sum()
+            loss.backward()
+            optimizer.step()
+
+            with torch.no_grad():
+                if iter_ % save_best_every == 0 or iter_ == num_iter - 1:
+                    # Evaluate the optimized locations and pick the best one.
+                    log_probs_of_optimized = potential_fn(tf_inv(optimize_inits))
+                    best_theta_iter = optimize_inits[
+                        torch.argmax(log_probs_of_optimized)
+                    ]
+                    best_log_prob_iter = potential_fn(tf_inv(best_theta_iter))
+                    if best_log_prob_iter > best_log_prob_overall:
+                        best_theta_overall = best_theta_iter.detach().clone()
+                        best_log_prob_overall = best_log_prob_iter.detach().clone()
+
+                if show_progress_bars:
+                    print(
+                        f"""Optimizing MAP estimate. Iterations: {iter_+1} /
+                        {num_iter}. Performance in iteration
+                        {divmod(iter_+1, save_best_every)[0] * save_best_every}:
+                        {best_log_prob_iter.item():.2f} (= unnormalized log-prob""",
+                        end="\r",
+                    )
+                argmax_ = tf_inv(best_theta_overall)
+                max_val = best_log_prob_overall
+
+            iter_ += 1
+
+    except KeyboardInterrupt:
+        interruption = f"Optimization was interrupted after {iter_} iterations. "
+        print(interruption + interruption_note)
+        return argmax_, max_val
+
+    return tf_inv(best_theta_overall), max_val
+
+
 def expit(theta_t: Tensor, lower_bound: Tensor, upper_bound: Tensor) -> Tensor:
     """
     Return the expit() of an input.
@@ -556,6 +884,67 @@ def within_support(distribution: Any, samples: Tensor) -> Tensor:
         return torch.isfinite(distribution.log_prob(samples))
 
 
+def mcmc_transform(
+    prior: Any,
+    num_prior_samples_for_zscoring: int = 1000,
+    enable_transform: bool = True,
+    device: str = "cpu",
+    **kwargs,
+) -> torch_tf.Transform:
+    """
+    Builds a transform that is applied to parameters during MCMC.
+
+    The resulting transform is defined such that the forward mapping maps from
+    constrained to unconstrained space.
+
+    It does two things:
+    1) When the prior support is bounded, it transforms the parameters into unbounded
+        space.
+    2) It z-scores the parameters such that MCMC is performed in a z-scored space.
+
+    Args:
+        prior: The prior distribution.
+        num_prior_samples_for_zscoring: The number of samples drawn from the prior
+            to infer the `mean` and `stddev` of the prior used for z-scoring. Unused if
+            the prior has bounded support or when the prior has `mean` and `stddev`
+            attributes.
+        enable_transform: Whether or not to use a transformation during MCMC.
+
+    Returns: A transformation that transforms whose `forward()` maps from unconstrained
+        (or z-scored) to constrained (or non-z-scored) space.
+    """
+
+    if enable_transform:
+        if hasattr(prior.support, "base_constraint") and hasattr(
+            prior.support.base_constraint, "upper_bound"
+        ):
+            transform = biject_to(prior.support)
+        else:
+            if hasattr(prior, "mean") and hasattr(prior, "stddev"):
+                prior_mean = prior.mean.to(device)
+                prior_std = prior.stddev.to(device)
+            else:
+                theta = prior.sample((num_prior_samples_for_zscoring,))
+                prior_mean = theta.mean(dim=0).to(device)
+                prior_std = theta.std(dim=0).to(device)
+
+            transform = torch_tf.AffineTransform(loc=prior_mean, scale=prior_std)
+    else:
+        transform = torch_tf.identity_transform
+
+    # Pytorch `transforms` do not sum the determinant over the parameters. However, if
+    # the `transform` explicitly is an `IndependentTransform`, it does. Since our
+    # `BoxUniform` is a `Independent` distribution, it will also automatically get a
+    # `IndependentTransform` wrapper in `biject_to`. Our solution here is to wrap all
+    # transforms as `IndependentTransform`.
+    if not isinstance(transform, torch_tf.IndependentTransform):
+        transform = torch_tf.IndependentTransform(
+            transform, reinterpreted_batch_ndims=1
+        )
+
+    return transform.inv
+
+
 class ImproperEmpirical(Empirical):
     """
     Wrapper around pyro's `Emprirical` distribution that returns constant `log_prob()`.
@@ -584,10 +973,7 @@ class ImproperEmpirical(Empirical):
 
 
 def mog_log_prob(
-    theta: Tensor,
-    logits_pp: Tensor,
-    means_pp: Tensor,
-    precisions_pp: Tensor,
+    theta: Tensor, logits_pp: Tensor, means_pp: Tensor, precisions_pp: Tensor
 ) -> Tensor:
     r"""
     Returns the log-probability of parameter sets $\theta$ under a mixture of Gaussians.
