@@ -16,14 +16,13 @@ from sbi.types import Array, OneOrMore, ScalarFloat
 
 
 def process_device(device: str, prior: Optional[Any] = None) -> str:
-    """Set and return the default device to cpu or gpu.
+    """Set and return the default device to cpu or cuda.
 
     Throws an AssertionError if the prior is not matching the training device not.
     """
 
     if not device == "cpu":
-        if device == "gpu":
-            device = "cuda"
+        assert device.startswith("cuda"), f"Invalid device string: {device}."
         try:
             torch.zeros(1).to(device)
             warnings.warn(
@@ -245,7 +244,7 @@ class BoxUniform(Independent):
             device: device of the prior, defaults to "cpu", should match the training
                 device when used in SBI.
         """
-
+        device = process_device(device)
         super().__init__(
             Uniform(
                 low=torch.as_tensor(low, dtype=torch.float32, device=device),
